; ROT13 Server example
; View with tabsize = 4
; Part of the Winsock networking tutorial by Thomas Bleeker
; Visit www.MadWizard.org
;
.586
.model flat, stdcall

option casemap:none

include <windows.inc>
include <kernel32.inc>
include <user32.inc>
include <crtlib.inc>
include <ws2_32.inc>	; the winsock 2 include

includelib <kernel32.lib>
includelib <user32.lib>
includelib <crtlib.lib>
includelib <ws2_32.lib> ; the winsock 2 library

main					proto stdcall :dword, :dword
GetHostDescription		proto stdcall :dword
SetServerSockAddr		proto stdcall :dword, :dword
RunServer				proto stdcall :dword
HandleConnection		proto stdcall :dword, :dword
rot13					proto stdcall :dword, :dword

.data
	CR	equ 0Dh
	LF	equ	0Ah
	
	DEFAULT_PORT		equ	4444
	REQ_WINSOCK_VER		equ	2
	TEMP_BUFFER_SIZE 	equ 128
		
	g_msgCreateSock		db	"Creating socket... ",0
	g_msgCreated		db	"created.",CR,LF,0
	g_msgInitWinsock	db	"Initializing winsock... ",0
	g_msgInitialized	db	"initialized.",CR,LF,0
	g_msgDone			db	"done.",CR,LF,0
	g_msgCleanup		db	"Cleaning up winsock... ",0
	g_msgBindSocket		db	"Binding socket... ",0
	g_msgBound			db	"bound.",CR,LF,0
	g_msgListen			db	"Putting socket in listening mode... ",0
	g_msgWaitConn		db	"Waiting for incoming connection... ",0
	g_msgAccepted		db	"accepted.",CR,LF,0
	g_msgDesc			db	"Connected with %s.",CR,LF,0
	g_msgConnClosed		db	"Connection closed.",CR,LF,0
	
	g_fmtSockDesc		db	"%s:%u",0
	
	g_errCreateSock		db	"could not create socket.",CR,LF,0
	g_errSend			db	"socket error while sending.",CR,LF,0
	g_errRead			db	"socket error while receiving.",CR,LF,0
	g_errStartup		db	"startup failed!",0
	g_errVersion		db	"required version not supported!",0
	g_errCleanup		db	"cleanup failed!",CR,LF,0
	g_errBind			db	"could not bind socket.",CR,LF,0
	g_errListen			db	"could not put socket in listening mode.",CR,LF,0
	g_errAccept			db	"accept function failed.",CR,LF,0
	
.code

;-------------------------------------------------------------------------------
; 	startup code
;-------------------------------------------------------------------------------
start:
	sub		esp, 12
	lea		eax, [esp+0]	; &env
	lea		ecx, [esp+4]	; &argc
	lea		edx, [esp+8]	; &argv
	invoke	getmainargs, ecx, edx, eax, 0
	add		esp, 4			; remove env (not used)
	call	main
	invoke	ExitProcess, eax

;-------------------------------------------------------------------------------
; <rot13>
;-------------------------------------------------------------------------------
;   Performs ROT13 'encryption' on a given buffer
; Parameters
;	pBuffer		pointer to the buffer to rot13
;   len			length of the buffer		
; Return value 
;	Nothing
rot13 proc pBuffer:dword, len:dword
	push esi
	mov	 esi, [pBuffer]
	mov	 ecx, [len]
	test ecx, ecx
	jz   _done

	align 16
_char:
	movzx eax, byte ptr [esi]
	cmp	  eax, 'A'
	jb	  _next
	cmp	  eax, 'N'
	jb	  _p13
	cmp	  eax, 'Z'
	jbe	  _m13

	cmp	  eax, 'a'
	jb	  _next
	cmp	  eax, 'n'
	jb	  _p13
	cmp	  eax, 'z'
	ja	  _next
_m13:
	sub	  eax, 13
	jmp	  _write
_p13:
	add	  eax, 13 
_write:
	mov	  [esi], al
_next:
	inc esi
	dec ecx
	jnz _char
_done:
	pop	 esi
ret
rot13 endp

;-------------------------------------------------------------------------------
; <GetHostDescription>
;-------------------------------------------------------------------------------
; Parameters
;	pSockAddr		pointer to the sockaddr_in to get a description for
; Return value 
;  	The description as a string pointer. The memory this pointer points to
; 	should be freed with the CRT free function.
GetHostDescription proc uses ebx esi edi pSockAddr:dword
	
	; Allocate description string space
	invoke	malloc, 22 ; enough bytes for "xxx.xxx.xxx.xxx:yyyyy",0
	mov		edi, eax
	
	mov		ebx, [pSockAddr]
	movzx	eax, [ebx][sockaddr_in.sin_port]
	invoke	ntohs, eax
	movzx	esi, ax ; port number
	
	; Get IP in dotted string format
	invoke	inet_ntoa, [ebx][sockaddr_in.sin_addr.S_un.S_addr]
	
	; Format string
	invoke 	sprintf, edi, addr g_fmtSockDesc, eax, esi
	mov		eax, edi
ret
GetHostDescription endp


;-------------------------------------------------------------------------------
; <SetServerSockAddr>
;-------------------------------------------------------------------------------
; Parameters
;	pSockAddr		pointer to the sockaddr_in structure to fill
;	portNumber		server port number
; Return value 
;  	Nothing
SetServerSockAddr proc pSockAddr:dword, portNumber:dword

	mov		edx, [pSockAddr]
	mov		ecx, [portNumber]
	xchg 	cl, ch	; convert to network byte order
	
	mov		[edx][sockaddr_in.sin_family], AF_INET	
	mov		[edx][sockaddr_in.sin_port], cx
	mov		[edx][sockaddr_in.sin_addr.S_un.S_addr], INADDR_ANY
		
	ret
SetServerSockAddr endp	

;-------------------------------------------------------------------------------
; <HandleConnection>
;-------------------------------------------------------------------------------
; Parameters
;  	hClientSocket	handle to the client socket
;	pSockAddr		pointer to the sockaddr_in identifying the client
; Return value 
;  	1 on success, 0 on failure
HandleConnection proc uses ebx esi hClientSocket:dword, pSockAddr:dword
	
	local 	tempBuffer[TEMP_BUFFER_SIZE]:byte
	
	mov		esi, [hClientSocket]

	; Print description (IP:port) of connected client
	invoke 	GetHostDescription, [pSockAddr]
	mov		ebx, eax
	invoke 	printf, addr g_msgDesc, eax
	invoke	free, ebx ; free memory occupied by string from GetHostDescription

	; Read data
_readData:
		invoke 	recv, esi, addr tempBuffer, sizeof tempBuffer, 0
		test	eax, eax
		mov		ecx, offset g_errRead
		jz		_connectionClosed	; return value 0 means connection closed
		cmp		eax, SOCKET_ERROR
		je		_error

		mov 	ebx, eax
		; ebx/eax is the number of bytes received.
		; rot13 the data and send it back to the client
		invoke	rot13, addr tempBuffer, eax
		invoke	send, esi, addr tempBuffer, ebx, 0
		mov		ecx, offset g_errSend
		cmp		eax, SOCKET_ERROR
		je		_error
	
	jmp		_readData
	
_connectionClosed:
	
	invoke	printf, addr g_msgConnClosed
	
	xor		eax, eax
	inc		eax ; return code (1 = no error)
	ret

_error:
	invoke	printf, ecx
	xor		eax, eax
	ret
HandleConnection endp

;-------------------------------------------------------------------------------
; <RunServer>
;-------------------------------------------------------------------------------
; Parameters
;	portNumber		server port number
; Return value 
;  	1 on success, 0 on failure
RunServer proc uses ebx portNumber:dword
	local	hSocket:dword, 
			hClientSocket:dword,
			sockAddr:sockaddr_in,
			clientSockAddr:sockaddr_in,
			clientSockSize:dword
;			bSuccess:dword = ebx,
			
	
	mov	[hSocket], INVALID_SOCKET
	mov [hClientSocket], INVALID_SOCKET
	mov	ebx, 1 ; success return code
		
	; Create socket:
	invoke	printf, addr g_msgCreateSock
	invoke	socket, AF_INET, SOCK_STREAM, IPPROTO_TCP
	mov		ecx, offset g_errCreateSock
	cmp		eax, INVALID_SOCKET
	je		_error
	mov		[hSocket], eax
	invoke	printf, addr g_msgCreated
	
	; Bind socket
	invoke 	printf, addr g_msgBindSocket
	invoke 	SetServerSockAddr, addr sockAddr, [portNumber]
	invoke	bind, [hSocket], addr sockAddr, sizeof sockAddr
	test	eax, eax
	mov		ecx, offset g_errBind
	jnz		_error
	invoke	printf, addr g_msgBound
	
	; Put socket in listening mode
	invoke	printf, addr g_msgListen
	invoke	listen, [hSocket], SOMAXCONN
	mov		ecx, offset g_errListen
	jnz		_error
	invoke	printf, addr g_msgDone
	
	; Wait for connection
	invoke printf, addr g_msgWaitConn
		
	; Accept connection
	mov 	[clientSockSize], sizeof clientSockAddr
	invoke	accept, [hSocket], addr clientSockAddr, addr clientSockSize
	cmp		eax, INVALID_SOCKET
	mov		ecx, offset g_errAccept
	je		_error
	mov		[hClientSocket], eax
	invoke	printf, addr g_msgAccepted
	
	; Wait for and accept a connection:
	invoke	HandleConnection, [hClientSocket], addr clientSockAddr
	mov		ebx, eax ; pass return code from HandleConnection

_cleanup:
	; close socket if it was created:
	mov		eax, [hSocket]
	cmp		eax, INVALID_SOCKET
	je		@F
		invoke	closesocket, eax
	@@:
	
	; close client socket if it was set:
	mov		eax, [hClientSocket]
	cmp		eax, INVALID_SOCKET
	je		@F
		invoke	closesocket, eax
	@@:
	
	
	mov		eax, ebx
	ret

_error:
	invoke	printf, ecx
	xor		ebx, ebx  ; return code (0 = error)
	jmp		_cleanup
	
ret
RunServer endp

;-------------------------------------------------------------------------------
; <main>
;-------------------------------------------------------------------------------
main proc uses ebx argc:dword, argv:dword
	local 	wsaData:WSADATA
	
	invoke	printf, addr g_msgInitWinsock
	invoke	WSAStartup, REQ_WINSOCK_VER, addr wsaData
	mov		ecx, offset g_errStartup
	test	eax, eax
	jnz		_error
	
		; Check if major version (low byte) is at least REQ_WINSOCK_VER
		mov		byte ptr [wsaData.wVersion], REQ_WINSOCK_VER
		mov		ecx, offset g_errVersion
		jb		_error_cleanup
		
		invoke	printf, addr g_msgInitialized
		
		; Check if port number is given as the program's parameter,
		; otherwise use the default port number.
		mov		eax, DEFAULT_PORT
		cmp		[argc], 2		; at least 1 argument?
		mov		ecx, [argv]		; get argument vector
		jb		@F
			mov		eax, [ecx][1*4]	; get first argument
			invoke	atoi, eax		; convert string into an int
	@@:
		invoke RunServer, eax
		
		mov		ebx, eax
		xor		ebx, 1
		; ebx now is 0 on success, 1 on failure of RunServer
		
_cleanup:
	invoke	printf, addr g_msgCleanup
	invoke	WSACleanup
	test	eax, eax
	jz		_done
	invoke	printf, addr g_errCleanup
_done:
	invoke	printf, addr g_msgDone
	mov		eax, ebx	; return code in ebx
	ret	
	
_error_cleanup:
	mov		ebx, _cleanup
	jmp		_printError
_error:
	mov		ebx, _done
_printError:
	invoke	printf, ecx
	mov		eax, ebx
	mov		ebx, 1		; return 1 (error)
	jmp		eax
main endp

end start